--
-- Migrate GH Tester database from schema version 1.9.19 to 1.9.20
--
DECLARE 
  required_version VARCHAR2(100);
  current_version VARCHAR2(100);
BEGIN
  SELECT '1.9.19' INTO required_version FROM dual;
  SELECT value INTO current_version FROM db_details 
    WHERE property='schema.version.number' 
    AND ctime = (
      SELECT MAX(ctime) FROM db_details WHERE property='schema.version.number' 
    );
    
   IF current_version != required_version THEN
     RAISE_APPLICATION_ERROR( -20001, 
                             'Incorrect schema version: ' || current_version  ||
                             ', expected: ' || required_version );
                             
   ELSE
-- 
-- Update Script here
--
EXECUTE IMMEDIATE
'CREATE TABLE "EXECUTION_NOTE" (
    "EXECUTION_ID" NUMBER NOT NULL, 
    "NOTE"         CLOB   NOT NULL, 
    CONSTRAINT "EXECUTION_NOTE_PK" PRIMARY KEY ("EXECUTION_ID"), 
    FOREIGN KEY ("EXECUTION_ID")
    REFERENCES "EXECUTION" ("ID") ON DELETE CASCADE
)';

EXECUTE IMMEDIATE 'CREATE SEQUENCE "PERSON_SEQ"';

EXECUTE IMMEDIATE
'CREATE TABLE "PERSON" (
    "ID"       NUMBER       NOT NULL, 
    "USERNAME" VARCHAR2(32) NOT NULL, 
    CONSTRAINT "PERSON_PK" PRIMARY KEY ("ID"),
    CONSTRAINT "PERSON_USERNAME" UNIQUE ("USERNAME")
)';

EXECUTE IMMEDIATE
'CREATE TABLE "EXECUTION_LOCK" (
    "EXECUTION_ID" NUMBER NOT NULL, 
    "PERSON_ID"    NUMBER NOT NULL, 
    "TIME"         NUMBER(20, 0)    NOT NULL,
    CONSTRAINT "EXECUTION_LOCK_PK" PRIMARY KEY ("EXECUTION_ID"), 
    FOREIGN KEY ("EXECUTION_ID")
    REFERENCES "EXECUTION" ("ID"), 
    FOREIGN KEY ("PERSON_ID")
    REFERENCES "PERSON" ("ID")
)';

EXECUTE IMMEDIATE
'CREATE TABLE "EXECUTION_LABEL" (
    "EXECUTION_ID" NUMBER         NOT NULL, 
    "LABEL"        VARCHAR2(4000) NOT NULL, 
    CONSTRAINT "EXECUTION_LABEL_PK" PRIMARY KEY ("EXECUTION_ID"), 
    FOREIGN KEY ("EXECUTION_ID")
    REFERENCES "EXECUTION" ("ID")
)';

EXECUTE IMMEDIATE
'CREATE OR REPLACE FORCE VIEW "SCENARIO_EXECUTIONS" ("EXECUTION_ID", "LABEL", "LOCKBY", "LOCKDATE", "NOTES_PK", "PROJECT_PK", "SUITE_PK", "SUITE_ID", "SUITE_NAME", "SCENARIO_NAME", "START_TIME", "END_TIME") AS 
  SELECT sce.execution_id, l.label, u.username, u.time, n.execution_id, t.project_id, t.id, t.uniqueid, t.display, s.display, e.start_time, e.end_time
  FROM scenario s, scenario_execution sce, execution e
  LEFT OUTER JOIN execution_label l ON e.id = l.execution_id
  LEFT OUTER JOIN (
    SELECT sue.execution_id, app.id, app.uniqueid, app.display, app.project_id
    FROM suite_execution sue, appmodelitem app
    WHERE sue.appmodelitem_id = app.id
  ) t ON e.id = t.execution_id
  LEFT OUTER JOIN (
    SELECT el.execution_id, p.username, el.time
    FROM execution_lock el, person p
    WHERE el.person_id = p.id
  ) u ON e.id = u.execution_id
  LEFT OUTER JOIN execution_note n ON e.id = n.execution_id
  WHERE e.id = sce.execution_id AND sce.scenario_id = s.id';

INSERT INTO db_details VALUES
   ( (SELECT sysdate FROM dual), 'schema.version.number','1.9.20');

commit;

   END IF; -- version check
END;
/